@@ -1976,6 +1976,90 @@ export default function waitUntil(callback, options = {}) {
         );
     }
 
+    #[test]
+    fn class_static_blocks_with_line_breaks_01() {
+        let src = "class Foo {
+            static
+            {
+                1 + 1;
+            }
+        }";
+        assert_eq_ignore_span!(
+            test_parser(
+                src,
+                Syntax::Es(EsConfig {
+                    static_blocks: true,
+                    ..Default::default()
+                }),
+                |p| p.parse_expr()
+            ),
+            Box::new(Expr::Class(ClassExpr {
+                ident: Some(Ident {
+                    span,
+                    sym: "Foo".into(),
+                    optional: false,
+                }),
+                class: Class {
+                    span,
+                    decorators: Vec::new(),
+                    super_class: None,
+                    type_params: None,
+                    super_type_params: None,
+                    is_abstract: false,
+                    implements: Vec::new(),
+                    body: vec!(ClassMember::StaticBlock(StaticBlock {
+                        span,
+                        body: BlockStmt {
+                            span,
+                            stmts: vec!(stmt("1 + 1;")),
+                        }
+                    }))
+                }
+            }))
+        );
+    }
+
+    #[test]
+    fn class_static_blocks_with_line_breaks_02() {
+        let src = "class Foo {
+            static
+            {}
+        }";
+        assert_eq_ignore_span!(
+            test_parser(
+                src,
+                Syntax::Es(EsConfig {
+                    static_blocks: true,
+                    ..Default::default()
+                }),
+                |p| p.parse_expr()
+            ),
+            Box::new(Expr::Class(ClassExpr {
+                ident: Some(Ident {
+                    span,
+                    sym: "Foo".into(),
+                    optional: false,
+                }),
+                class: Class {
+                    span,
+                    decorators: Vec::new(),
+                    super_class: None,
+                    type_params: None,
+                    super_type_params: None,
+                    is_abstract: false,
+                    implements: Vec::new(),
+                    body: vec!(ClassMember::StaticBlock(StaticBlock {
+                        span,
+                        body: BlockStmt {
+                            span,
+                            stmts: Vec::new(),
+                        }
+                    }))
+                }
+            }))
+        );
+    }
+
     #[test]
     fn class_static_blocks_in_ts() {
         let src = "class Foo { static { 1 + 1 }; }";
@@ -1984,6 +2068,30 @@ export default function waitUntil(callback, options = {}) {
         });
     }
 
+    #[test]
+    fn class_static_blocks_with_line_breaks_in_ts_01() {
+        let src = "class Foo {
+            static
+            {
+                1 + 1;
+            }
+        }";
+        test_parser(src, Syntax::Typescript(Default::default()), |p| {
+            p.parse_expr()
+        });
+    }
+
+    #[test]
+    fn class_static_blocks_with_line_breaks_in_ts_02() {
+        let src = "class Foo {
+            static
+            {}
+        }";
+        test_parser(src, Syntax::Typescript(Default::default()), |p| {
+            p.parse_expr()
+        });
+    }
+
     #[test]
     #[should_panic(expected = "Modifiers cannot appear here")]
     fn class_static_blocks_in_ts_with_invalid_modifier_01() {
